#include <stdio.h>
#include <math.h>

#include "nrutil.h"

int IMIN(int ia, int ib) 
	{
	if (ia<=ib) 
		return ia;
	else 
		return ib;
	}

double DMAX(double a, double b) 
{
  if (a>=b) return a;
  else return b;
}

  void lubksb(double **a, int n, int *indx, double b[]);
  void ludcmp(double **a, int n, int *indx, double *d);

  void savgol(double *c, int np, int nl, int nr, int ld, int m)  {
/*------------------------------------------------------------------------------------------- 
 USES lubksb,ludcmp given below. 
 Returns in c(np), in wrap-around order (see reference) consistent with the argument respns 
 in routine convlv, a set of Savitzky-Golay filter coefficients. nl is the number of leftward 
 (past) data points used, while nr is the number of rightward (future) data points, making 
 the total number of data points used nl +nr+1. ld is the order of the derivative desired 
 (e.g., ld = 0 for smoothed function). m is the order of the smoothing polynomial, also 
 equal to the highest conserved moment; usual values are m = 2 or m = 4. 
-------------------------------------------------------------------------------------------*/
int imj,ipj,j,k,kk,mm,*indx;
double d,fac,sum,**a,*b;

if (np < nl+nr+1 || nl < 0 || nr < 0 || ld > m || nl+nr < m)
	nrerror("bad args in savgol");

indx=ivector(1,m+1);
a=matrix(1,m+1,1,m+1);
b=vector(1,m+1);
for (ipj=0;ipj<=(m << 1);ipj++) 
	{//Set up the normal equations of the desired least-squares fit
	sum=(ipj ? 0.0 : 1.0); 
	for (k=1;k<=nr;k++) 
		sum += pow((double)k,(double)ipj);
	for (k=1;k<=nl;k++) 
		sum += pow((double)-k,(double)ipj);
	mm=IMIN(ipj,2*m-ipj);
	for (imj = -mm;imj<=mm;imj+=2)
		a[1+(ipj+imj)/2][1+(ipj-imj)/2]=sum;
	}

ludcmp(a,m+1,indx,&d); //Solve them: LU decomposition.

for (j=1;j<=m+1;j++) 
	b[j]=0.0;

b[ld+1]=1.0; //Right-hand side vector is unit vector, depending on which derivative we want.

lubksb(a,m+1,indx,b); //Get one row of the inverse matrix.

for (kk=1;kk<=np;kk++) 
	c[kk]=0.0; //Zero the output array (it may be bigger than number of coefficients).

for (k = -nl;k<=nr;k++) 
	{ 
	sum=b[1];   //Each Savitzky-Golay coefficient is the dot product 
				//of powers of an integer with the inverse matrix row.
	fac=1.0;
	for (mm=1;mm<=m;mm++) 
		sum += b[mm+1]*(fac *= k);

	kk=((np-k) % np)+1; //Store in wrap-around order.
	c[kk]=sum;
	}

free_vector(b,1,m+1);
free_matrix(a,1,m+1,1,m+1);
free_ivector(indx,1,m+1);
}

/**************************************************************
* Given an N x N matrix A, this routine replaces it by the LU *
* decomposition of a rowwise permutation of itself. A and N   *
* are input. INDX is an output vector which records the row   *
* permutation effected by the partial pivoting; D is output   *
* as -1 or 1, depending on whether the number of row inter-   *
* changes was even or odd, respectively. This routine is used *
* in combination with LUBKSB to solve linear equations or to  *
* invert a matrix. Return code is 1, if matrix is singular.   *
**************************************************************/
void ludcmp(double **a, int n, int *indx, double *d)
{
int i,imax,j,k;
double big,dum,sum,temp;
double *vv=vector(1,n);
*d=1.0; 
for (i=1;i<=n;i++) 
	{
	big=0.0;
	for (j=1;j<=n;j++)
		if ((temp=fabs(a[i][j])) > big) 
			big=temp;
	if (big == 0.0) 
			nrerror("allocation failure 1 in matrix()");	
	vv[i]=1.0/big; 
	}
for (j=1;j<=n;j++) 
	{ 
	for (i=1;i<j;i++) 
		{ 
		sum=a[i][j];
		for (k=1;k<i;k++) 
			sum -= a[i][k]*a[k][j];
		a[i][j]=sum;
		}
	big=0.0; 
	for (i=j;i<=n;i++) 
		{ 
		sum=a[i][j];
		for (k=1;k<j;k++)
			sum -= a[i][k]*a[k][j];
		a[i][j]=sum;
		if ( (dum=vv[i]*fabs(sum)) >= big) 
			{
			big=dum;
			imax=i;
			}
		}
	if (j != imax) 
		{ 
		for (k=1;k<=n;k++) 
			{ 
			dum=a[imax][k];
			a[imax][k]=a[j][k];
			a[j][k]=dum;
			}
		*d = -(*d); 
		vv[imax]=vv[j]; 
		}
	indx[j]=imax;
	if (a[j][j] == 0.0) 
		a[j][j]=TINY;

	if (j != n) 
		{ 
		dum=1.0/(a[j][j]);
		for (i=j+1;i<=n;i++) 
			a[i][j] *= dum;
		}
	} 
free_vector(vv,1,n);
}

/*****************************************************************
* Solves the set of N linear equations A . X = B.  Here A is    *
* input, not as the matrix A but rather as its LU decomposition, *
* determined by the routine LUDCMP. INDX is input as the permuta-*
* tion vector returned by LUDCMP. B is input as the right-hand   *
* side vector B, and returns with the solution vector X. A, N and*
* INDX are not modified by this routine and can be used for suc- *
* cessive calls with different right-hand sides. This routine is *
* also efficient for plain matrix inversion.                     *
*****************************************************************/
void lubksb(double **a, int n, int *indx, double b[])
{
int i,ii=0,ip,j;
double sum;

for (i=1;i<=n;i++) 
	{ 
	ip=indx[i];
	sum=b[ip];
	b[ip]=b[i];
	if (ii)
		for (j=ii;j<=i-1;j++) 
			sum -= a[i][j]*b[j];
	else if (sum) 
		ii=i; 

	b[i]=sum;
	}
for (i=n;i>=1;i--) 
	{
	sum=b[i];
	for (j=i+1;j<=n;j++) 
		sum -= a[i][j]*b[j];

	b[i]=sum/a[i][i];
	} 
} 


void four1(double *d, int nn, int isign) { 
/*------------------------------------------------------------------------------------------- 
 Replaces d(1:2*nn) by its discrete Fourier transform, if isign is input as 1; or replaces 
 d(1:2*nn) by nn times its inverse discrete Fourier transform, if isign is input as -1. 
 d is a complex array of length nn or, equivalently, a real array of length 2*nn. nn 
 MUST be an integer power of 2 (this is not checked for here). 
--------------------------------------------------------------------------------------------*/
//Labels: e1, e2
  int    i,istep,j,m,mmax,n; 
  double  tempi,tempr; 
  double theta,wi,wpi,wpr,wr,wtemp;  //Double precision for the trigonometric recurrences
     
  n=2*nn;
  j=1;
  i=1;
  while (i<=n) {  //This is the bit-reversal section of the routine
	if (j > i) {
      tempr=d[j];        //Exchange the two complex numbers
      tempi=d[j+1];
      d[j]=d[i];
      d[j+1]=d[i+1];
      d[i]=tempr;
      d[i+1]=tempi;
    } 
    m=nn;
e1: if (m >=2 && j > m) {
      j=j-m;
      m=m/2;
      goto e1; 
    } 
    j=j+m;
    i += 2;
  } 
  mmax=2;                     //Here begins the Danielson-Lanczos section of the routine
e2:if (n > mmax) {                                   //Outer loop executed log2 nn times
    istep=2*mmax;
    theta=6.28318530717959/(isign*mmax);   //Initialize for the trigonometric recurrence
    wpr=-2.0*sin(0.5*theta)*sin(0.5*theta);
    wpi=sin(theta);
    wr=1.0;
    wi=0.0;
    m=1;
    while (m<=mmax) {                              //Here are the two nested inner loops
      i=m;
      while (i<=n) {
        j=i+mmax;                                //This is the Danielson-Lanczos formula:
        tempr=wr*d[j]-wi*d[j+1];
        tempi=wr*d[j+1]+wi*d[j];
        d[j]=d[i]-tempr;
        d[j+1]=d[i+1]-tempi;
        d[i]=d[i]+tempr;
        d[i+1]=d[i+1]+tempi;
        i += istep;
      }
      wtemp=wr;  //Trigonometric recurrence
      wr=wr*wpr-wi*wpi+wr;
      wi=wi*wpr+wtemp*wpi+wi;
      m += 2;
    } 
    mmax=istep;
    goto e2;  //Not yet done
   }           //All done 
} 

void realft(double *d, int n, int isign) {
/*-------------------------------------------------------------------------------------------
 USES four1
 Calculates the Fourier transform of a set of n real-valued d points. Replaces this d 
 (which is stored in array d(1:n)) by the positive frequency half of its complex Fourier 
 transform. The real-valued first and last components of the complex transform are returned 
 as elements d(1) and d(2), respectively. n must be a power of 2. This routine 
 also calculates the inverse transform of a complex d array if it is the transform of real 
 d. (Result in this case must be multiplied by 2/n.) 
-------------------------------------------------------------------------------------------*/
  int    i,i1,i2,i3,i4,n2p3; 
  double  c1,c2,h1i,h1r,h2i,h2r,wis,wrs;
  double theta,wi,wpi,wpr,wr,wtemp;    //Double precision for the trigonometric recurrences
  double PI=4.0*atan(1.0);   
  
  theta=PI/(n/2);                      //Initialize the recurrence
  c1=0.5;
  if (isign == 1) {
    c2=-0.5;
    four1(d,n/2,1);                 //The forward transform is here
  }   
  else {
    c2=0.5;                            //Otherwise set up for an inverse transform
    theta=-theta;
  }
  wpr=-2.0*sin(0.5*theta)*sin(0.5*theta);
  wpi=sin(theta);
  wr=1.0+wpr;
  wi=wpi;
  n2p3=n+3;
  for (i=2; i<= n/4; i++) {            //Case i=1 done separately below
    i1=2*i-1;
    i2=i1+1;
    i3=n2p3-i2;
    i4=i3+1;
    wrs=wr;
    wis=wi;
    h1r=c1*(d[i1]+d[i3]);    //The two separate transforms are separated out of d
    h1i=c1*(d[i2]-d[i4]);
    h2r=-c2*(d[i2]+d[i4]);
    h2i=c2*(d[i1]-d[i3]);
    d[i1]=h1r+wrs*h2r-wis*h2i;    //Here they are recombined to form the true transform
                                      //of the original real d
    d[i2]=h1i+wrs*h2i+wis*h2r;
    d[i3]=h1r-wrs*h2r+wis*h2i;
    d[i4]=-h1i+wrs*h2i+wis*h2r;
    wtemp=wr;                         //The recurrence
    wr=wr*wpr-wi*wpi+wr;
    wi=wi*wpr+wtemp*wpi+wi;
  }

  if (isign == 1) {
    h1r=d[1];
    d[1]=h1r+d[2];
    d[2]=h1r-d[2];         //Squeeze the first and last d together to get
  }                              //them all within the original array
  else {
    h1r=d[1];
    d[1]=c1*(h1r+d[2]);
    d[2]=c1*(h1r-d[2]);
    four1(d,n/2,-1);          //This is the inverse transform for the case isign-1
  } 
}

void smoothFT(double *y, int n, int points) {
//-------------------------------------------------------------------------------------------------------------------------
// Smooths an array "y" of length "n", with a window whose full width
// is of order "points" neighboring points, a user supplied value.  Array y is modified.
//--------------------------------------------------------------------------------------------------------------------------
int   i, k, MO2;
double CNST,f,RN1,Y1,YN;
double *s;
int m=2;
int nmin=n + 2*points;
while (m < nmin) 
    m*=2;
m*=2;

CNST=(double)(points*points)/(double)(m*m);
  
s= vector(1, m);
for (i=1; i<=n; i++)     
    s[i]=y[i-1];
  
Y1=s[1];
YN=s[n];
RN1=1.0/(n-1);
   
for (i=1; i<=n; i++)//Remove linear trend
	s[i]-=RN1*(Y1*(n-i)+YN*(i-1));

if (n+1 <= m)
	{
    for (i=n+1; i<=m; i++)  
		s[i]=0.0;
	}

MO2=m/2;
realft(s, MO2, 1);//Fourier transform

s[1]=s[1]/MO2;
f=1.0;
for (i=1; i<MO2; i++) 
	{
    k=2*i+1;
	f=DMAX(0.0,(1.0-CNST*i*i)/MO2);
    if (f != 0.0) 
		{
      	s[k]=f*s[k];
      	s[k+1]=f*s[k+1];
    	}
    else 
		{
      	s[k]=0.0;
      	s[k+1]=0.0;
    	}
  	}
	
f=DMAX(0.0,(1.0-0.25*points*points)/MO2);  //Last point
s[2]=f*s[2];

realft(s,MO2,-1);//Inverse Fourier transform
  
for (i=1; i<=n; i++)//Restore linear trend
	s[i]+=RN1*(Y1*(n-i)+YN*(i-1));

for (i=1; i<=n; i++)     
    y[i-1]=s[i];
free_vector(s, 1, m);
}
